#ifndef BBTREE_H
#define BBTREE_H

#include "bbnode.h"

#include <iostream>
#include <vector>
#include <algorithm>


//cf. declaration of pop_heap: returns the element which returns false if compared to all other elements in the heap
struct HigherBound { // pop_heap returns the node with the LOWEST bound
	template< typename Node >
	bool operator()( Node* a, Node* b )
	{
		return a->bound > b->bound;
	}
};

struct LowerBound { //pop_heap returns the element with the HIGHEST bound
	template< typename Node >
	bool operator()( Node* a, Node* b )
	{
		return a->bound < b->bound;
	}
};

struct bbTree {
	typedef bbNode Node;

	bool foundFeasibleSolution;
	double bestKnownIntegralSolution;

	Node* root;

	std::vector<int> incumbent;

	std::vector< Node* > nextNodes;

	unsigned int num_nodes;

	bbTree();
	~bbTree();

	void new_best_solution( const double& s )
	{
		foundFeasibleSolution = true;
		bestKnownIntegralSolution = s;
	}

	void commit_incumbent( std::vector<int> new_incumbent_ ){
		incumbent = new_incumbent_;
	}

	std::vector<int>* get_incumbent() {
		return &incumbent;
	}

	bool worse( const double& b ) const
	{
		return foundFeasibleSolution && 
			b > bestKnownIntegralSolution  + 1e-10;
	}

	void push_node( Node* node )
	{
		nextNodes.push_back( node );
		std::push_heap( nextNodes.begin(), nextNodes.end(), HigherBound() );
	}

	Node* get_next_node()
	{
		if( nextNodes.empty() ) return 0;
		Node* next = nextNodes.front();	
		std::pop_heap( nextNodes.begin(), nextNodes.end(), HigherBound() );
		nextNodes.pop_back();
		return next;
	}

	void propagate_bound( Node* node )
	{
		Node* current = node;
		while( current/* && changed*/ ){
			Node* lower = current->lowerChild;
			Node* upper = current->upperChild;
			double childBound = 0.0;
			if(lower && upper){
				if(lower->feasible){
					if( upper->feasible )
						childBound = min( lower->bound, upper->bound);
					else
						childBound = lower->bound;
				}else{
					if(upper->feasible)
						childBound = upper->bound;
				}
				if(childBound >= current->bound)
					current->bound = childBound;
			}
			current = current->parent;
		}
	}

	double min (double a, double b){
		return a < b ? a : b;
	}

};

#endif
