#include "qsopt.h"
#include "bbnode.h"
#include "bbtree.h"

#include <iostream>
#include <iterator>

using namespace std;

double frac( const double& f )
{
	return f - trunc( f );
}

double fractionallity( int ncols, double* x, int* var )
{
	*var = 0;
	double sum = frac( x[0] ) * ( 1 - frac( x[0] ) );
	double maxff = sum;
	for ( int i = 0; i < ncols; ++i ) {
		const double f = frac( x[i] );
		const double ff = f * ( 1- f );
		sum += ff;
		if ( ff > maxff ) {
			*var = i;
			maxff = ff;
		}
	}

	return sum;
}

int main( int argc, char** argv ) {
	
	const double eps = 1e-10;

	bbTree B;
	bbNode* N = new bbNode;

	B.root = N;

	if ( argc != 3 ) {
		cerr << "Usage: " << argv[0] << " <filename> <format>" << endl;
		exit(0);
	}

	QSprob p = QSread_prob( argv[1], argv[2] );
	if ( p == (QSprob) NULL ) {
		cerr << "Unable to read and load problem " << argv[1] 
				 << " of type " << argv[2] << endl;
		exit(0);
	}
	const int ncols = QSget_colcount( p );

	QSprob cut_prob = QSread_prob( argv[1], argv[2] );
	if ( cut_prob == (QSprob) NULL ) {
		cerr << "Unable to read and load problem " << argv[1] 
				 << " of type " << argv[2] << endl;
		exit(0);
	}
	const int nr = QSget_rowcount( cut_prob );
	int allrows[nr];
	for ( int i = 0; i < nr; ++i )
		allrows[i] = i;
	//	QSdelete_rows( cut_prob, nr - 1, allrows );
	//	bool cut_unbounded = true;
	//	double cut_threshold = 0;
	//	double cut_frx = ncols;
	//	int ncuts = nr;
	//	QSchange_objsense( cut_prob, QS_MAX );

	//QSbas basis;

	//	QSset_param( p, QS_PARAM_DUAL_PRICING, QS_PRICE_DDANTZIG );

	int status;
	double x[ncols];
	double frx;
	unsigned int nnodes = 1;
	int obj_bound_idx = 0;
	double obj[ncols];
	QSget_obj( p, obj );
	int objnzcnt = 0;
	int objcolidx[ncols];
	double objcolval[ncols];
	for ( int i = 0; i < ncols; ++i ) 
		if ( obj[i] != 0 ) {
			objcolidx[objnzcnt] = i;
			objcolval[objnzcnt] = obj[i];
			++objnzcnt;
		}


	do {
		// all work done for this node?
		if ( N->lowerChild && N->upperChild ) 
			if ( N == B.root ) break;
			else {
				N = N->parent;
				QSchange_bound( p, N->variable, 'U', N->upperBound );
				QSchange_bound( p, N->variable, 'L', N->lowerBound );
				
				//				QSopt_primal( p, &status );

				continue;
			}
		if ( !N->lowerChild && !N->upperChild ) {
			QSopt_dual( p, &status );

			if( status == QS_LP_INFEASIBLE ) {
				if ( N == B.root ) {
					cout << "root infeasible" << endl;
					break;
				}
				N->feasible = false;
				const int nrows = QSget_rowcount( p );
				double pi[nrows];
				QSget_infeas_array( p, pi );
				if ( nrows % 100 == 0 )
					cout << nrows << endl;
				//				copy( pi, x+nrows, ostream_iterator<double>( cout, " " ) );
				//				cout << endl;
				bbNode* PN = N->parent;
				bbNode* HN = B.root;
				int iscnt = 0;
				int cutcolidx[ncols];
				double cutcolval[ncols];
				double cutrhs = 0.0;
				while( PN ) {
					int* colidx = NULL;
					double* colval = NULL;
					int* colcnt = NULL;
					double* lower = NULL;
					double* upper = NULL;
					int collist[1] = { PN->variable };
					QSget_columns_list( p, 1, collist, &colcnt, NULL,
															&colidx, &colval,
															NULL, &lower, &upper, NULL );
					double s = 0.0;
					for ( int i = 0; i < colcnt[0]; ++i ) 
						s += pi[ colidx[i] ] * colval[i];
					if ( fabs( s ) > eps ) {
						if ( upper[0] < 1 ) {
							cutcolval[iscnt] = 1;
							//							cutrhs += upper[0];
						} else {
							cutcolval[iscnt] = -1;
							cutrhs += -1;
							//							cutrhs += -lower[0];
						}

						cutcolidx[iscnt] = PN->variable;
						++iscnt;
						HN = PN;
					}
					PN = PN->parent;
					QSfree( colidx );
					QSfree( colval );
					QSfree( colcnt );
					QSfree( lower  );
					QSfree( upper  );
				}

#if 0
					// delete obsolete parts of the tree
					bbNode* QN = N->parent;
					do {
						QSchange_bound( p, QN->variable, 'U', QN->upperBound );
						QSchange_bound( p, QN->variable, 'L', QN->lowerBound );
						QN = QN->parent;
					} while( QN != HN->parent );
					bbNode* NN = new bbNode;
					++nnodes;
					if ( HN->parent ) {
						if ( HN == HN->parent->upperChild )
							HN->parent->upperChild = NN;
						else HN->parent->lowerChild = NN;
						NN->parent = HN->parent;
					} else B.root = NN;
					N = NN;
					
					delete HN;
#else
					N = N->parent;
					QSchange_bound( p, N->variable, 'U', N->upperBound );
					QSchange_bound( p, N->variable, 'L', N->lowerBound );
#endif
					//				QSload_basis( p, basis );
					//				QSopt_primal( p, &status );

				// add cut
#if 1
				if ( iscnt <= ncols * 0.2 + 2 ) {
					QSadd_row( p, iscnt, cutcolidx, cutcolval, cutrhs + 1, 'G', NULL );
					//					QSopt_dual( p, &status );
				}
#else
				QSadd_row( cut_prob, iscnt, cutcolidx, cutcolval,
									 cutrhs + 1, 'L', NULL );
				++ncuts;
				QSopt_dual( cut_prob, &status );
				bool cut_keep = false;
				if ( status == QS_LP_OPTIMAL ) {
					double threshold; 
					double frx;
					double x[ncols];
					QSget_x_array( cut_prob, x );
					int var;
					frx = fractionallity( ncols, x, &var );
					QSget_objval( cut_prob, &threshold );
					if ( cut_unbounded || 
							 // iscnt < 0.17 * ncols + 2 ||
							 frx < cut_frx ||
							 threshold > cut_threshold ) {
						cout << ncuts << " " << threshold << " " << frx << endl;
						QSadd_row( p, iscnt, cutcolidx, cutcolval, cutrhs + 1, 'G', NULL );
						//						QSopt_dual( p, &status );
						cut_threshold = threshold;
						cut_unbounded = false;
						//						cut_keep = true;
						cut_frx = frx;
					} 
				}
				if ( !cut_keep ) {
					QSdelete_row( cut_prob, ncuts - 1 );
					--ncuts;
					QSopt_primal( cut_prob, &status );
				}
#endif

				continue;
			} else {
				//				basis = QSget_basis( p );
				QSget_objval( p, &N->bound );

				if ( B.foundFeasibleSolution && 
						 N->bound > B.bestKnownIntegralSolution - 1 + ncols * eps ) {
					double rc[ncols];
					QSget_rc_array( p, rc );
					bbNode* PN = N->parent;
					int iscnt = 0;
					int cutcolidx[ncols];
					double cutcolval[ncols];
					double cutrhs = 0.0;
					while( PN ) {
						double upper;
						QSget_bound( p, PN->variable, 'U', &upper );
						const double s = rc[PN->variable];
						if ( s < -eps && upper < 1 ) {
							cutcolval[iscnt] = 1;
							//							cutrhs += upper[0];
							cutcolidx[iscnt] = PN->variable;
							++iscnt;
						} else if ( s > eps && upper > 0 ) {
							cutcolval[iscnt] = -1;
							cutrhs -= 1;
							//							cutrhs += -lower[0];
							cutcolidx[iscnt] = PN->variable;
							++iscnt;
						}

						PN = PN->parent;
					}
					N = N->parent;
					QSchange_bound( p, N->variable, 'U', N->upperBound );
					QSchange_bound( p, N->variable, 'L', N->lowerBound );

					//					QSopt_primal( p, &status );

					// add cut
					if ( iscnt > 0 && iscnt <= ncols * 0.2 + 2 ) {
						cout << iscnt << endl;
						QSadd_row( p, iscnt, cutcolidx, cutcolval, cutrhs + 1, 'G', NULL );
					}

					continue;
				}

				QSget_x_array( p, x );
				int var;
				frx = fractionallity( ncols, x, &var );
				
				if ( frx < ncols * eps ) {
					cout << "integral solution " 
							 << N->bound << " found after " << nnodes << " nodes" << endl;
					//					copy( x, x+ncols, ostream_iterator<double>( cout, " " ) );
					//					cout << endl;
					if( !B.foundFeasibleSolution ) {
						B.foundFeasibleSolution = true;
						B.bestKnownIntegralSolution = (int) round( N->bound );
						QSadd_row( p, objnzcnt, objcolidx, objcolval,
											 B.bestKnownIntegralSolution - 1 , 'L', "obj_bound" );
						QSget_row_index( p, "obj_bound", &obj_bound_idx );
					} else if ( N->bound < B.bestKnownIntegralSolution ) {
						B.bestKnownIntegralSolution = (int) round( N->bound );
						QSchange_rhscoef( p, obj_bound_idx, 
															B.bestKnownIntegralSolution - 1 );
					}

					//					QSopt_dual( p, &status );

					N = N->parent;
					QSchange_bound( p, N->variable, 'U', N->upperBound );
					QSchange_bound( p, N->variable, 'L', N->lowerBound );
					
					//					QSopt_primal( p, &status );
				 
					continue;
				}

				N->variable = var;
				N->value = x[var];

				double lb, ub;
				QSget_bound( p, var, 'L', &lb );
				QSget_bound( p, var, 'U', &ub );
				N->lowerBound = (int) lb;
				N->upperBound = (int) ub;

				// branch for lower or upper bound
				if ( frac( x[var] ) > 0.5 ) { // upper
					++nnodes;
					bbNode* NN = new bbNode;
					N->upperChild = NN;
					NN->parent = N;
					N = NN;
					QSchange_bound( p, var, 'U', floor( x[var] ) );
				} else { // lower
					++nnodes;
					bbNode* NN = new bbNode;
					N->lowerChild = NN;
					NN->parent = N;
					N = NN;
					QSchange_bound( p, var, 'L', ceil( x[var] ) );
				}
			}
			continue;
		}

		if ( !N->lowerChild ) {
			++nnodes;
			bbNode* NN = new bbNode;
			N->lowerChild = NN;
			NN->parent = N;
			QSchange_bound( p, N->variable, 'L', ceil( N->value ) );
			QSchange_bound( p, N->variable, 'U', N->upperBound );
			N = NN;
		} else {
			++nnodes;
			bbNode* NN = new bbNode;
			N->upperChild = NN;
			NN->parent = N;
			QSchange_bound( p, N->variable, 'U', floor( N->value ) );
			QSchange_bound( p, N->variable, 'L', N->lowerBound );
			N = NN;
		}
		
	} while( true );

	cout << "best integral solution: " << B.bestKnownIntegralSolution 
			 << std::endl;
	cout << "number of nodes in bb-tree: " << nnodes << endl;

	QSwrite_prob( cut_prob, "cutprob.lp", "LP" );

	QSfree_prob( p );
	QSfree_prob( cut_prob );
	//	QSfree_basis( basis );

	delete B.root;

	return 0;
}
