#ifndef _INFEASIBILITYHANDLER_H_
#define _INFEASIBILITYHANDLER_H_

#include "noop.h"

#include <iostream>

template< int num_solvers, class Tree, class Solver, class Oracle = SayYes >
struct FarkasLearner {

	Solver* solver;
	Oracle oracle;
	
	FarkasLearner( Tree& t, Solver s[num_solvers] ) 
		: solver( s ), oracle( t, s ) {}

	template< typename Node >
	void operator()( const int& s, Node* node )
	{
		Node* traverse = node->parent;
		solver[s].compute_iis();
		for( int i = 0; i < num_solvers; ++i )
		  solver[i].cut_begin( 1 );
		int cut_count = 0;
		int depth = 0;
		while( traverse ) {
			if ( solver[s].member_of_iis( traverse->variable ) && 
					 oracle( traverse ) ) {
				if ( solver[s].upper_bound_of( traverse->variable ) < 1 )
					for( int i = 0; i < num_solvers; ++i )
						solver[i].cut_add( traverse->variable, 1, 0 );
				else
					for( int i = 0; i < num_solvers; ++i )
						solver[i].cut_add( traverse->variable, -1, -1 );
				++cut_count;
			}
			++depth;
			traverse = traverse->parent;
		}

		if ( depth != cut_count && cut_count < 11 ) {
			std::cout << cut_count << " " << depth << std::endl;
			for( int i = 0; i < num_solvers; ++i )
  			solver[i].cut_commit();
		}
	
	}
	
};

template< class Tree, class Solver >
struct KickOutEitherWay {

	Tree& tree;
	Solver& solver;
	bool skip;

	KickOutEitherWay( Tree& t, Solver& s ) 
		: tree( t ), solver( s ), skip( false ) {}

	template< typename Node >
	bool operator()( Node* node )
	{
		if ( skip ) return true;
		skip = true;
		const int variable = node->variable;
		const double lb = solver.lower_bound_of( node->variable );
		const double ub = solver.upper_bound_of( node->variable );
		
		solver.set_bounds( variable, 1 - lb, 1 - ub );
		solver.solve();
		solver.set_bounds( variable, lb, ub );
		if( solver.lp_infeasible() ) return false;
		if( solver.lp_feasible() ) 
			if ( tree.worse( solver.objective() ) ) return false;
		skip = false;
		return true;
	}

};

#endif 
