#ifndef _INTEGRALITYHANDLER_H_
#define _INTEGRALITYHANDLER_H_

#include <iostream>

template< class Tree >
struct NotifyTreeOnNewIntegralSolution {
	
	Tree& tree;

	template< class Solver >
	NotifyTreeOnNewIntegralSolution( Tree& t, Solver& s ) : tree( t ) {}

	template< typename Node >
	void operator()( Node* node )
	{
		const double obj = node->bound;
		std::cout << "New best integral solution " << obj 
							<< " found after " << tree.num_nodes << " nodes"
							<< std::endl;
		tree.new_best_solution( obj );
	}

};

template< class Tree >
struct NotifyTreeOnPossiblyNewIncumbent {

	Tree& tree;

	template< class Solver >
	NotifyTreeOnPossiblyNewIncumbent( Tree& t, Solver& s ) : tree( t ) {}

	template< typename Node >
	void operator()( Node* node, double obj )
	{
		node->integral = true;
		if(tree.worse(obj)){
			return;
		}

		tree.new_best_solution( obj );
	}

};

template< class Tree, class Solver >
struct NotifyAndCutIntegralSolution {
	
	Tree& tree;
	Solver& solver;

	NotifyAndCutIntegralSolution( Tree& t, Solver& s ) 
		: tree( t ), solver( s ) {}

	template< typename Node >
	void operator()( Node* node )
	{
		const int obj = (int) round( node->bound );
		if( !tree.foundFeasibleSolution ) 
			solver.create_objective_cut( obj - 1 );
		else
			solver.refine_objective_cut( obj - 1 );
		tree.new_best_solution( obj );
	}

};

#endif 
