#ifndef MATUTILS_H
#define MATUTILS_H

#include <iostream>
#include <vector>
#include <sstream>
#include <math.h>
#include <assert.h>
#include <limits>

int bits( int n ) {
  int w = 1;
  while( n > (1<<w) ) {
    ++w;
  }
  return w;
}

std::string name( const std::string& prefix, int l, int j ) {
  std::stringstream ss;
  ss << prefix << "(" << l << "," << j << ")";
  return ss.str();
}

std::string name(std::string prefix, int b, int i, int j){
		std::stringstream ss;
		ss << prefix << "(" << b << "," << i << "," << j << ")";
		return ss.str();
	}

std::string name( const std::string& prefix, int k, int i, int l, int j ) {
  std::stringstream ss;
  if( l < k ) {
    ss << prefix << "(" << k << "," << i << "," << l << "," << j << ")";
  } else {
    ss << prefix << "(" << l << "," << j << "," << k << "," << i << ")";
  }     
  return ss.str();
}

typedef std::vector< std::vector<double> > Matrix;

Matrix emptyMatrix( int n ) {
  return Matrix( n, std::vector<double>( n ) );
}

Matrix emptyMatrix( int b, int n ) {
  return Matrix( b, std::vector<double>( n ) );
}

void readMatrix( Matrix& M, std::istream &in ) {
  for( auto& r : M ) {
    for( auto& a : r ) {
      in >> a;
    }
  }
}

template< typename T >
void readVector( std::vector<T>& v, std::istream &in ) {
  for( auto& a : v ) {
    in >> a;
  }
}

void transpose( const Matrix& M, Matrix& T ) {
  unsigned int n = M.size();
  for( unsigned int i = 0; i < n; ++i ) {
    for( unsigned int j = 0; j < n; ++j ) {
      T[j][i] = M[i][j];
    }
  }
}

std::vector<int> binarydecomposition( int i, int w ) {
  std::vector<int> ret( w );
  for( int b = 0; b < w; ++b ) {
    const int j = i >> b;
    ret[b] = j%2;
  }
  return ret;
}

struct LetterSlot {
  unsigned int letter;
  unsigned int slot;
};

std::vector<LetterSlot> readFixations(std::istream &in) {
  unsigned int ff;
  in >> ff;
  const unsigned int f = ff;
   
  std::vector<LetterSlot> F;
  for( unsigned int i = 0; i < f; ++i ) {
    unsigned int l,j;
    in >> l;
    in >> j;
    F.push_back( {l,j} );
  }
  return F;
}

void printFixations( const std::vector<LetterSlot>& F ) {
  for( auto f : F ) {
    std::cout << name( "x", f.letter, f.slot ) << " = 1" << std::endl;
  }
}

template< typename T >
void printVector(const std::vector<T> v){
	for(auto const& d : v){
		std::cout << d << " ";
	}
	std::cout << std::endl;
}

void printMatrix( Matrix& M ) {
  for( auto& r : M ) {
    for( auto& a : r ) {
      std::cout << a << " ";
    }
   std::cout << std::endl;
  }
}

void scale( Matrix& M , std::istream &in) {
  double scale;
  in >> scale;
  
  const unsigned int n = M.size();
  
  for( unsigned int l = 0; l < n; ++l ) {
    for( unsigned int k = 0; k < n; ++k ) {
      M[l][k] = floor( M[l][k]*scale )/scale;
    }
  }
}

void ignoreComment(std::istream &in) {
  if( (in >> std::ws ).peek() == std::char_traits<char>::to_int_type('#') ) {
    in.ignore( std::numeric_limits<std::streamsize>::max(), '\n' );
  }
}

#endif
