#ifndef _SDPTRAITS_H_
#define _SDPTRAITS_H_


template< int num_solvers,
					typename Tree,
					typename Solver,
					typename InfeasibilityHandler,
					typename WorsenessHandler,
					typename IntegralityHandler,
					typename HeuristicHandler>
struct SdpTraits {
	typedef typename Tree::Node Node;

	Tree tree;

	Solver solver[num_solvers];

	InfeasibilityHandler node_infeasible_notification;
	WorsenessHandler node_worse_notification;
	IntegralityHandler node_integral_notification;
	HeuristicHandler produce_heuristic_solution;


	SdpTraits() 
		: node_infeasible_notification( tree, solver ),
			 node_worse_notification( tree, solver ),
			 node_integral_notification( tree, solver ),
			 produce_heuristic_solution( tree, solver[0] )
	{
		for(int i = 0; i < num_solvers; ++i){
			solver[i].setID(i);
		}
	}

	bool lower_first( const int& i ) 
	{
		return solver[i].value_of( solver[i].get_candidate() ) < 0;
	}

};

#endif
