#ifndef _SOLVER_H_
#define _SOLVER_H_

struct Solver {

	//! reads the problem 
	/*! \param filename the name of the file to be read
		  \param format indicates the format ( "MPS", "LP" )
	*/
	void read( char* filename, char* format );

	//! solves the problem in the current state
	void solve();
	
	//! sets lower and upper bounds of a given variable
	void set_bounds( const unsigned int& variable,
									 const double& lowerBound,
									 const double& upperBound );

	//! reports wether the last call to solve led to infeasibility
	bool lp_infeasible();

	//! reports wether the last call to solve led to feasibility
	bool lp_feasible();

	//! reports wether the last call to solve led to integrality
	bool lp_integral();

	//! returns the last objective value
	double objective();

	//! returns the value of a given variable in the last optimal solution
	double value_of( const unsigned int& variable );

	//! returns the lower bound of a given variable
	double lower_bound_of( const unsigned int& variable );

	//! returns the upper bound of a given variable
	double upper_bound_of( const unsigned int& variable );

	//! returns a variable for a branching candidate
	int get_candidate();

	//! computes an iis in case of infeasibility
	void compute_iis();

	//! reports wether a given variable is in the latest computed iis
	bool member_of_iis( const int& variable );

	//! begins an empty cut with given right-hand-side
	void cut_begin( const double& rhs );

	//! adds a variable with its coefficient and increment of the rhs to the cut
	void cut_add( const int& variable, const double& coef, const double& rhs );
	
	//! commits the cut to the solver
	void cut_commit();
	
};

#endif
